package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.base.entity.BasePowerLineWellheadEnt;
import pps.core.base.entity.BaseWellheadEnt;
import pps.core.base.mapper.BasePowerLineWellheadMapper;
import pps.core.base.mapper.BaseWellheadMapper;
import pps.core.base.service.data.base_wellhead.*;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;
import java.util.Objects;


/**
 * 井口配置模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BaseWellheadService extends BaseService {

    @XText("新增")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseWellhead(XContext context, CreateBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        BaseWellheadEnt entity = new BaseWellheadEnt();
        XCopyUtils.copyObject(input, entity);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseUtils.setBaseModelDefault(entity, session);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("更新")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseWellhead(XContext context, UpdateBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseWellheadEnt::getId, input.getId());
        BaseWellheadEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseUtils.setBaseModelDefault(entity, session);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseWellhead(XContext context, DeleteBaseWellheadInput input) {
        String wellheadId = input.getId();
        if (this.checkWellheadIsReference(context, wellheadId)) {
            return XServiceResult.error(992, "当前井口已被引用");
        }
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseWellheadEnt::getId, wellheadId);
        BaseWellheadEnt entity = mapper.selectOne(queryWrapper);
        if (Objects.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        entity = new BaseWellheadEnt();
        entity.setId(input.getId());
        entity.setIsDeleted(BusinessConstant.ZERO);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseUtils.setBaseModelDefault(entity, session);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("根据ID获取详情")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseWellheadOutput> getBaseWellhead(XContext context, GetBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BaseWellheadEnt::getId, input.getId())
                .eq(BaseWellheadEnt::getIsDeleted, BusinessConstant.ONE);
        BaseWellheadEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseWellheadOutput output = new GetBaseWellheadOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页查询")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseWellheadOutput> queryBaseWellhead(XContext context, QueryBaseWellheadInput input) {
        String excludeIdFlag = input.getExcludeIdFlag();
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        List<String> allListByOuId = this.getAllListByOuId(context, input.getOuId());
        queryWrapper.lambda()
                .notExists("1".equals(excludeIdFlag), "SELECT 1 FROM base_power_line_wellhead w WHERE w.ou_id = {0} AND base_wellhead.id = w.wellhead_id", input.getOuId())
                .eq(BaseWellheadEnt::getIsDeleted, BusinessConstant.ONE)
                .in(CollUtil.isNotEmpty(allListByOuId), BaseWellheadEnt::getOuId, allListByOuId)
                .orderByDesc(BaseWellheadEnt::getModifyTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseWellheadOutput.class);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查井口是否被引用
     *
     * @param context    上下文
     * @param wellheadId 井口ID
     * @return boolean
     */
    private boolean checkWellheadIsReference(XContext context, String wellheadId) {
        BasePowerLineWellheadMapper mapper = context.getBean(BasePowerLineWellheadMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLineWellheadEnt>()
                .eq(BasePowerLineWellheadEnt::getWellheadId, wellheadId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }
}
